@@ -1,30 +1,44 @@
-import pyomo.environ as pyo
 import numpy as np
+import pyomo.environ as pyo
 
 from omlt.formulation import _PyomoFormulation, _setup_scaled_inputs_outputs
-from omlt.neuralnet.layer import ConvLayer, DenseLayer, InputLayer
-from omlt.neuralnet.layers.full_space import (full_space_dense_layer, full_space_conv_layer)
+from omlt.neuralnet.activations import (
+    ACTIVATION_FUNCTION_MAP as _DEFAULT_ACTIVATION_FUNCTIONS,
+)
+from omlt.neuralnet.activations import (
+    ComplementarityReLUActivation,
+    bigm_relu_activation_constraint,
+    linear_activation_constraint,
+    linear_activation_function,
+    sigmoid_activation_constraint,
+    sigmoid_activation_function,
+    softplus_activation_constraint,
+    softplus_activation_function,
+    tanh_activation_constraint,
+    tanh_activation_function,
+)
+from omlt.neuralnet.layer import ConvLayer2D, DenseLayer, InputLayer, PoolingLayer2D
+from omlt.neuralnet.layers.full_space import (
+    full_space_conv2d_layer,
+    full_space_dense_layer,
+    full_space_maxpool2d_layer,
+)
+from omlt.neuralnet.layers.partition_based import (
+    default_partition_split_func,
+    partition_based_dense_relu_layer,
+)
 from omlt.neuralnet.layers.reduced_space import reduced_space_dense_layer
-from omlt.neuralnet.layers.partition_based import partition_based_dense_relu_layer, default_partition_split_func
-from omlt.neuralnet.activations import (linear_activation_constraint,
-                                        linear_activation_function,
-                                        bigm_relu_activation_constraint,
-                                        ComplementarityReLUActivation,
-                                        sigmoid_activation_constraint,
-                                        softplus_activation_constraint,
-                                        tanh_activation_constraint,
-                                        sigmoid_activation_function,
-                                        softplus_activation_function,
-                                        tanh_activation_function)
-from omlt.neuralnet.activations import ACTIVATION_FUNCTION_MAP as _DEFAULT_ACTIVATION_FUNCTIONS
+
 
 def _ignore_input_layer():
     pass
 
+
 _DEFAULT_LAYER_CONSTRAINTS = {
     InputLayer: _ignore_input_layer,
     DenseLayer: full_space_dense_layer,
-    ConvLayer: full_space_conv_layer,
+    ConvLayer2D: full_space_conv2d_layer,
+    PoolingLayer2D: full_space_maxpool2d_layer,
 }
 
 _DEFAULT_ACTIVATION_CONSTRAINTS = {
@@ -32,9 +46,10 @@ _DEFAULT_ACTIVATION_CONSTRAINTS = {
     "relu": bigm_relu_activation_constraint,
     "sigmoid": sigmoid_activation_constraint,
     "softplus": softplus_activation_constraint,
-    "tanh": tanh_activation_constraint
+    "tanh": tanh_activation_constraint,
 }
 
+
 class FullSpaceNNFormulation(_PyomoFormulation):
     """
     This class is the entry-point to build neural network formulations.
@@ -52,7 +67,10 @@ class FullSpaceNNFormulation(_PyomoFormulation):
     activation_constraints : dict-like or None
         overrides the constraints generated for the specified activation functions
     """
-    def __init__(self, network_structure, layer_constraints=None, activation_constraints=None):
+
+    def __init__(
+        self, network_structure, layer_constraints=None, activation_constraints=None
+    ):
         super().__init__()
 
         self.__network_definition = network_structure
@@ -60,7 +78,9 @@ class FullSpaceNNFormulation(_PyomoFormulation):
         self.__scaled_input_bounds = network_structure.scaled_input_bounds
 
         self._layer_constraints = dict(self._supported_default_layer_constraints())
-        self._activation_constraints = dict(self._supported_default_activation_constraints())
+        self._activation_constraints = dict(
+            self._supported_default_activation_constraints()
+        )
         if layer_constraints is not None:
             self._layer_constraints.update(layer_constraints)
         if activation_constraints is not None:
@@ -68,21 +88,25 @@ class FullSpaceNNFormulation(_PyomoFormulation):
 
         # TODO: Change these to exceptions.
         network_inputs = list(self.__network_definition.input_nodes)
-        assert len(network_inputs) == 1, 'Multiple input layers are not currently supported.'
+        assert (
+            len(network_inputs) == 1
+        ), "Multiple input layers are not currently supported."
         network_outputs = list(self.__network_definition.output_nodes)
-        assert len(network_outputs) == 1, 'Multiple output layers are not currently supported.'
+        assert (
+            len(network_outputs) == 1
+        ), "Multiple output layers are not currently supported."
 
     def _supported_default_layer_constraints(self):
         return _DEFAULT_LAYER_CONSTRAINTS
 
     def _supported_default_activation_constraints(self):
         return _DEFAULT_ACTIVATION_CONSTRAINTS
-    
+
     def _build_formulation(self):
-        _setup_scaled_inputs_outputs(self.block,
-                                     self.__scaling_object,
-                                     self.__scaled_input_bounds)
-        
+        _setup_scaled_inputs_outputs(
+            self.block, self.__scaling_object, self.__scaled_input_bounds
+        )
+
         _build_neural_network_formulation(
             block=self.block,
             network_structure=self.__network_definition,
@@ -94,18 +118,24 @@ class FullSpaceNNFormulation(_PyomoFormulation):
     def input_indexes(self):
         """The indexes of the formulation inputs."""
         network_inputs = list(self.__network_definition.input_nodes)
-        assert len(network_inputs) == 1, 'Multiple input layers are not currently supported.'
+        assert (
+            len(network_inputs) == 1
+        ), "Multiple input layers are not currently supported."
         return network_inputs[0].input_indexes
 
     @property
     def output_indexes(self):
         """The indexes of the formulation output."""
         network_outputs = list(self.__network_definition.output_nodes)
-        assert len(network_outputs) == 1, 'Multiple output layers are not currently supported.'
+        assert (
+            len(network_outputs) == 1
+        ), "Multiple output layers are not currently supported."
         return network_outputs[0].output_indexes
 
 
-def _build_neural_network_formulation(block, network_structure, layer_constraints, activation_constraints):
+def _build_neural_network_formulation(
+    block, network_structure, layer_constraints, activation_constraints
+):
     """
     Adds the neural network formulation to the given Pyomo block.
 
@@ -147,15 +177,23 @@ def _build_neural_network_formulation(block, network_structure, layer_constraint
             continue
         layer_id = id(layer)
         layer_block = block.layer[layer_id]
-        layer_constraints_func = layer_constraints.get(type(layer), None)
-        activation_constraints_func = activation_constraints.get(layer.activation, None)
 
+        layer_constraints_func = layer_constraints.get(type(layer), None)
         if layer_constraints_func is None:
-            raise ValueError('Layer type {} is not supported by this formulation.'.format(type(layer)))
-        if activation_constraints_func is None:
-            raise ValueError('Activation {} is not supported by this formulation.'.format(layer.activation))
-        
+            raise ValueError(
+                "Layer type {} is not supported by this formulation.".format(
+                    type(layer)
+                )
+            )
         layer_constraints_func(block, net, layer_block, layer)
+
+        activation_constraints_func = activation_constraints.get(layer.activation, None)
+        if activation_constraints_func is None:
+            raise ValueError(
+                "Activation {} is not supported by this formulation.".format(
+                    layer.activation
+                )
+            )
         activation_constraints_func(block, net, layer_block, layer)
 
     # setup input variables constraints
@@ -176,13 +214,15 @@ def _build_neural_network_formulation(block, network_structure, layer_constraint
 
     @block.Constraint(output_layer.output_indexes)
     def output_assignment(b, *output_index):
-        return b.scaled_outputs[output_index] == b.layer[id(output_layer)].z[output_index]
+        return (
+            b.scaled_outputs[output_index] == b.layer[id(output_layer)].z[output_index]
+        )
 
 
 class FullSpaceSmoothNNFormulation(FullSpaceNNFormulation):
     def __init__(self, network_structure):
         """
-        This class is used for building "full-space" formulations of 
+        This class is used for building "full-space" formulations of
         neural network models composed of smooth activations (e.g., tanh,
         sigmoid, etc.)
 
@@ -198,13 +238,14 @@ class FullSpaceSmoothNNFormulation(FullSpaceNNFormulation):
             "linear": linear_activation_constraint,
             "sigmoid": sigmoid_activation_constraint,
             "softplus": softplus_activation_constraint,
-            "tanh": tanh_activation_constraint
+            "tanh": tanh_activation_constraint,
         }
 
+
 class ReluBigMFormulation(FullSpaceNNFormulation):
     def __init__(self, network_structure):
         """
-        This class is used for building "full-space" formulations of 
+        This class is used for building "full-space" formulations of
         neural network models composed of relu activations using a
         big-M formulation
 
@@ -218,13 +259,14 @@ class ReluBigMFormulation(FullSpaceNNFormulation):
     def _supported_default_activation_constraints(self):
         return {
             "linear": linear_activation_constraint,
-            "relu": bigm_relu_activation_constraint
+            "relu": bigm_relu_activation_constraint,
         }
 
+
 class ReluComplementarityFormulation(FullSpaceNNFormulation):
     def __init__(self, network_structure):
         """
-        This class is used for building "full-space" formulations of 
+        This class is used for building "full-space" formulations of
         neural network models composed of relu activations using
         a complementarity formulation (smooth represenation)
 
@@ -238,7 +280,7 @@ class ReluComplementarityFormulation(FullSpaceNNFormulation):
     def _supported_default_activation_constraints(self):
         return {
             "linear": linear_activation_constraint,
-            "relu": ComplementarityReLUActivation()
+            "relu": ComplementarityReLUActivation(),
         }
 
 
@@ -254,6 +296,7 @@ class ReducedSpaceNNFormulation(_PyomoFormulation):
     activation_functions : dict-like or None
         overrides the actual functions used for particular activations
     """
+
     def __init__(self, network_structure, activation_functions=None):
         super().__init__()
 
@@ -261,21 +304,22 @@ class ReducedSpaceNNFormulation(_PyomoFormulation):
         self.__scaling_object = network_structure.scaling_object
         self.__scaled_input_bounds = network_structure.scaled_input_bounds
 
-        
         # TODO: look into increasing support for other layers / activations
-        #self._layer_constraints = {**_DEFAULT_LAYER_CONSTRAINTS, **layer_constraints}
-        self._activation_functions = dict(self._supported_default_activation_functions())
+        # self._layer_constraints = {**_DEFAULT_LAYER_CONSTRAINTS, **layer_constraints}
+        self._activation_functions = dict(
+            self._supported_default_activation_functions()
+        )
         if activation_functions is not None:
             self._activation_functions.update(activation_functions)
 
     def _supported_default_activation_functions(self):
         return dict(_DEFAULT_ACTIVATION_FUNCTIONS)
-    
+
     def _build_formulation(self):
-        _setup_scaled_inputs_outputs(self.block,
-                                     self.__scaling_object,
-                                     self.__scaled_input_bounds)
-        
+        _setup_scaled_inputs_outputs(
+            self.block, self.__scaling_object, self.__scaled_input_bounds
+        )
+
         net = self.__network_definition
         layers = list(net.layers)
         block = self.block
@@ -287,8 +331,10 @@ class ReducedSpaceNNFormulation(_PyomoFormulation):
         # currently only support a single input layer
         input_layers = list(net.input_layers)
         if len(input_layers) != 1:
-            raise ValueError('build_formulation called with a network that has more than'
-                             ' one input layer. Only single input layers are supported.')        
+            raise ValueError(
+                "build_formulation called with a network that has more than"
+                " one input layer. Only single input layers are supported."
+            )
         input_layer = input_layers[0]
         input_layer_id = id(input_layer)
         input_layer_block = block.layer[input_layer_id]
@@ -310,10 +356,14 @@ class ReducedSpaceNNFormulation(_PyomoFormulation):
             # build the linear expressions and the activation function
             layer_id = id(layer)
             layer_block = block.layer[layer_id]
-            layer_func = reduced_space_dense_layer #layer_constraints[type(layer)]
+            layer_func = reduced_space_dense_layer  # layer_constraints[type(layer)]
             activation_func = self._activation_functions.get(layer.activation, None)
             if activation_func is None:
-                raise ValueError('Activation {} is not supported by this formulation.'.format(layer.activation))
+                raise ValueError(
+                    "Activation {} is not supported by this formulation.".format(
+                        layer.activation
+                    )
+                )
 
             layer_func(block, net, layer_block, layer, activation_func)
 
@@ -321,14 +371,19 @@ class ReducedSpaceNNFormulation(_PyomoFormulation):
         # currently only support a single output layer
         output_layers = list(net.output_layers)
         if len(output_layers) != 1:
-            raise ValueError('build_formulation called with a network that has more than'
-                             ' one output layer. Only single output layers are supported.')
+            raise ValueError(
+                "build_formulation called with a network that has more than"
+                " one output layer. Only single output layers are supported."
+            )
         output_layer = output_layers[0]
 
         @block.Constraint(output_layer.output_indexes)
         def output_assignment(b, *output_index):
             pb = b.parent_block()
-            return b.scaled_outputs[output_index] == b.layer[id(output_layer)].z[output_index]
+            return (
+                b.scaled_outputs[output_index]
+                == b.layer[id(output_layer)].z[output_index]
+            )
 
     # @property
     # def layer_constraints(self):
@@ -342,16 +397,19 @@ class ReducedSpaceNNFormulation(_PyomoFormulation):
     def input_indexes(self):
         """The indexes of the formulation inputs."""
         network_inputs = list(self.__network_definition.input_nodes)
-        assert len(network_inputs) == 1, 'Unsupported multiple network input variables'
+        assert len(network_inputs) == 1, "Unsupported multiple network input variables"
         return network_inputs[0].input_indexes
 
     @property
     def output_indexes(self):
         """The indexes of the formulation output."""
         network_outputs = list(self.__network_definition.output_nodes)
-        assert len(network_outputs) == 1, 'Unsupported multiple network output variables'
+        assert (
+            len(network_outputs) == 1
+        ), "Unsupported multiple network output variables"
         return network_outputs[0].output_indexes
 
+
 class ReducedSpaceSmoothNNFormulation(ReducedSpaceNNFormulation):
     """
     This class is used to build reduced-space formulations
@@ -362,6 +420,7 @@ class ReducedSpaceSmoothNNFormulation(ReducedSpaceNNFormulation):
     network_structure : NetworkDefinition
         the neural network definition
     """
+
     def __init__(self, network_structure):
         super().__init__(network_structure)
 
@@ -370,8 +429,9 @@ class ReducedSpaceSmoothNNFormulation(ReducedSpaceNNFormulation):
             "linear": linear_activation_function,
             "sigmoid": sigmoid_activation_function,
             "softplus": softplus_activation_function,
-            "tanh": tanh_activation_function
-            }
+            "tanh": tanh_activation_function,
+        }
+
 
 class ReluPartitionFormulation(_PyomoFormulation):
     """
@@ -385,6 +445,7 @@ class ReluPartitionFormulation(_PyomoFormulation):
     split_func : callable
         the function used to compute the splits
     """
+
     def __init__(self, network_structure, split_func=None):
         super().__init__()
 
@@ -398,9 +459,9 @@ class ReluPartitionFormulation(_PyomoFormulation):
         self.__split_func = split_func
 
     def _build_formulation(self):
-        _setup_scaled_inputs_outputs(self.block,
-                                     self.__scaling_object,
-                                     self.__scaled_input_bounds)
+        _setup_scaled_inputs_outputs(
+            self.block, self.__scaling_object, self.__scaled_input_bounds
+        )
         block = self.block
         net = self.__network_definition
         layers = list(net.layers)
@@ -432,12 +493,16 @@ class ReluPartitionFormulation(_PyomoFormulation):
             layer_block = block.layer[layer_id]
             if isinstance(layer, DenseLayer):
                 if layer.activation == "relu":
-                    partition_based_dense_relu_layer(block, net, layer_block, layer, split_func)
+                    partition_based_dense_relu_layer(
+                        block, net, layer_block, layer, split_func
+                    )
                 elif layer.activation == "linear":
                     full_space_dense_layer(block, net, layer_block, layer)
                     linear_activation_constraint(block, net, layer_block, layer)
                 else:
-                    raise ValueError("ReluPartitionFormulation supports Dense layers with relu or linear activation")
+                    raise ValueError(
+                        "ReluPartitionFormulation supports Dense layers with relu or linear activation"
+                    )
             else:
                 raise ValueError("ReluPartitionFormulation supports only Dense layers")
 
@@ -449,7 +514,10 @@ class ReluPartitionFormulation(_PyomoFormulation):
 
         @block.Constraint(input_layer.output_indexes)
         def input_assignment(b, *output_index):
-            return b.scaled_inputs[output_index] == b.layer[id(input_layer)].z[output_index]
+            return (
+                b.scaled_inputs[output_index]
+                == b.layer[id(input_layer)].z[output_index]
+            )
 
         # setup output variables constraints
         # currently only support a single output layer
@@ -459,18 +527,23 @@ class ReluPartitionFormulation(_PyomoFormulation):
 
         @block.Constraint(output_layer.output_indexes)
         def output_assignment(b, *output_index):
-            return b.scaled_outputs[output_index] == b.layer[id(output_layer)].z[output_index]
+            return (
+                b.scaled_outputs[output_index]
+                == b.layer[id(output_layer)].z[output_index]
+            )
 
     @property
     def input_indexes(self):
         """The indexes of the formulation inputs."""
         network_inputs = list(self.__network_definition.input_nodes)
-        assert len(network_inputs) == 1, 'Unsupported multiple network input variables'
+        assert len(network_inputs) == 1, "Unsupported multiple network input variables"
         return network_inputs[0].input_indexes
 
     @property
     def output_indexes(self):
         """The indexes of the formulation output."""
         network_outputs = list(self.__network_definition.output_nodes)
-        assert len(network_outputs) == 1, 'Unsupported multiple network output variables'
+        assert (
+            len(network_outputs) == 1
+        ), "Unsupported multiple network output variables"
         return network_outputs[0].output_indexes
